//+------------------------------------------------------------------+
//|                                          WriteAllChartsToCSV.mq4 |
//|                                                               me |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "me"
#property link      ""

#property indicator_chart_window


extern int BarHistory=2000;


string FileName[5];  // 5m ,15m, 1h, 1d, 1w
int Periods[5] = { PERIOD_M5, PERIOD_M15, PERIOD_H1, PERIOD_D1, PERIOD_W1}; 
string FileNameEnd[5] = {"5m.csv","15m.csv","1h.csv","1d.csv","1w.csv"};
int Handles[5]; //5m ,15m, 1h, 1d, 1w  files

int TotalSymbols;
string SymbolArray[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
	TotalSymbols = StoreSymbols(SymbolArray);

	for(int i =0; i< TotalSymbols; i++){


		for(int j=0; j<5 ; j++ ){

			FileName[j] = StringSubstr(SymbolArray[i],0,6) + FileNameEnd[j] ;      //We are interested in 5 periods for each symbol

			FileDelete(FileName[j]);    //makes new files.  
			Handles[j]  = FileOpen(FileName[j] , FILE_CSV|FILE_READ|FILE_WRITE, "\t");

			if (Handles[j]<1){
				Print("cannot open file error_",GetLastError());
				return(0);
			}
		}

		string Strline;
		for (int cnt=BarHistory; cnt>0; cnt--){  //last bar not handled here

			for( j=0; j<5 ; j++ ){
				Strline = MakeInfoString(SymbolArray[i],Periods[j] ,cnt);
				FileWrite(Handles[j], Strline);
			}
		}   //we wrote 1999 lines in each file

		for( j=0; j<5 ; j++ )
			FileClose(Handles[j]);


	} //now next symbol


}   //At this point all the files should be filled with all data except last bar	

//+------------------------------------------------------------------+

    
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
	for( int j=0; j<5 ; j++ )       //last set of opened files
		FileClose(Handles[j]);
}
  
  
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{



	for(int i =0; i< TotalSymbols; i++){


		for(int j=0; j<5 ; j++ ){

			FileName[j] = StringSubstr(SymbolArray[i],0,6) + FileNameEnd[j] ;      //We are interested in 5 periods for each symbol
			
			Handles[j]  = FileOpen(FileName[j] , FILE_CSV|FILE_READ|FILE_WRITE, "\t");

			if (Handles[j]<1){
				Print("cannot open file error_",GetLastError());
				return(0);
			}
		}



		static string LastTickStrings[5];
		static double ClosePrices[5];   
		static datetime CurrentTimeStamps[5];

		//----------Check for new bar -----------\\
		for( j=0; j<5 ; j++ ){
			if(CurrentTimeStamps[j] != iTime(SymbolArray[i],Periods[j], 0) ){    //this conditon should be met on the first start() iteration

				CurrentTimeStamps[j] = iTime(SymbolArray[i],Periods[j], 0);

				//print a new line
				LastTickStrings[j] = MakeInfoString(SymbolArray[i],Periods[j] ,0);
				FileWrite(Handles[j], LastTickStrings[j]);
				ClosePrices[j] = iClose(SymbolArray[i],Periods[j] ,0);
			}	

			else if(ClosePrices[j] != iClose(SymbolArray[i],Periods[j] ,0) ){   //if not a newbar, but a new tick, we should delete the last line and rewrite		

				int ByteCount = StringLen(LastTickStrings[j]) + 2 ; //count number of bytes to go back

				LastTickStrings[j] = MakeInfoString(SymbolArray[i],Periods[j] ,0);
				FileSeek(Handles[j],-1*ByteCount, SEEK_END);
				FileWrite(Handles[j], LastTickStrings[j] );
			}
		}


		for( j=0; j<5 ; j++ )
			FileClose(Handles[j]);


	}  //next symbol



	return(0);

}
//+------------------------------------------------------------------+



string MakeInfoString(string symbol, int period, int Bar){

	string Strline = TimeToStr(iTime(symbol,period, Bar),TIME_DATE|TIME_SECONDS);

	Strline = StringSubstr(Strline,0,4) + StringSubstr(Strline,5,2) + StringSubstr(Strline,8,2) + "," + StringSubstr(Strline,11,5);

	Strline = Strline + "," + DoubleToStr(iOpen(symbol,period, Bar),4) + "," + DoubleToStr(iHigh(symbol,period, Bar),4) + "," 
		+ DoubleToStr(iLow(symbol,period, Bar),4) + "," + DoubleToStr(iClose(symbol,period, Bar),4) + "," + DoubleToStr(iVolume(symbol,period, Bar),0);

	return(Strline);
}



//Stores the symbols into the givin array and returns the number of symbols
int StoreSymbols(string& SymbolArray[]) 
{
	int handle=FileOpenHistory("symbols.raw", FILE_BIN | FILE_READ);
	if(handle<1)
	{
		Print("Error Opening File: ", GetLastError());
		return(false);
	}  


	int TotalRecords=FileSize(handle) / 1936;
	ArrayResize(SymbolArray, TotalRecords);


	string SymbolName;

	for(int i=0; i<TotalRecords; i++) 
	{
		SymbolName =FileReadString(handle, 12);
		FileSeek(handle, 1924, SEEK_CUR); // goto the next record

		SymbolArray[i]=SymbolName;

	}

	FileClose(handle);

	return(TotalRecords);
}